@@ -1,4 +1,6 @@
 import pprint
+from auto_schema.diff import Diff
+from auto_schema.change_type import ChangeType
 import pytest
 import os
 
@@ -9,6 +11,7 @@ from sqlalchemy.dialects import postgresql
 from . import conftest
 from auto_schema import runner
 from auto_schema import ops
+from auto_schema.clause_text import get_clause_text
 
 
 def get_new_metadata_for_runner(r):
@@ -206,9 +209,9 @@ def validate_column(schema_column, db_column, metadata, dialect):
 
 
 def validate_column_server_default(schema_column, db_column):
-    schema_clause_text = runner.Runner.get_clause_text(
+    schema_clause_text = get_clause_text(
         schema_column.server_default)
-    db_clause_text = runner.Runner.get_clause_text(db_column.server_default)
+    db_clause_text = get_clause_text(db_column.server_default)
 
     if isinstance(schema_column.type, sa.Boolean):
         schema_clause_text = runner.Runner.convert_postgres_boolean(
@@ -222,33 +225,46 @@ def validate_column_server_default(schema_column, db_column):
 
 
 def validate_column_type(schema_column, db_column, metadata, dialect):
-    # print(type(schema_column.type).__name__, schema_column.type, db_column.type, schema_column.type == db_column.type, str(schema_column.type) == str(db_column.type))
+    # array type. validate contents
+    if isinstance(schema_column.type, postgresql.ARRAY):
+        assert isinstance(db_column.type, postgresql.ARRAY)
 
-    if isinstance(schema_column.type, sa.TIMESTAMP):
+        validate_column_type_impl(
+            schema_column.type.item_type, db_column.type.item_type, metadata, dialect, db_column, schema_column)
+    else:
+
+        validate_column_type_impl(
+            schema_column.type, db_column.type, metadata, dialect, db_column, schema_column)
+    pass
+
+
+def validate_column_type_impl(schema_column_type, db_column_type, metadata, dialect, db_column, schema_column):
+
+    if isinstance(schema_column_type, sa.TIMESTAMP):
         # timezone not supported in sqlite so this is just ignored there
         if dialect != 'sqlite':
-            assert schema_column.type.timezone == db_column.type.timezone
+            assert schema_column_type.timezone == db_column_type.timezone
         else:
-            assert str(db_column.type) == "TIMESTAMP"
-    elif isinstance(schema_column.type, sa.Time):
+            assert str(db_column_type) == "TIMESTAMP"
+    elif isinstance(schema_column_type, sa.Time):
         # timezone not supported in sqlite so this is just ignored there
         if dialect != 'sqlite':
-            assert schema_column.type.timezone == db_column.type.timezone
+            assert schema_column_type.timezone == db_column_type.timezone
         else:
-            assert str(db_column.type) == "TIME"
-    elif isinstance(schema_column.type, sa.Numeric):
-        assert isinstance(db_column.type, sa.Numeric)
+            assert str(db_column_type) == "TIME"
+    elif isinstance(schema_column_type, sa.Numeric):
+        assert isinstance(db_column_type, sa.Numeric)
         # precision is tricky so ignore this for now
         # assert schema_column.type.precision == db_column.type.precision
-    elif isinstance(schema_column.type, postgresql.ENUM):
+    elif isinstance(schema_column_type, postgresql.ENUM):
         # enum type if possible otherwise check constraint...
-        assert isinstance(db_column.type, postgresql.ENUM)
+        assert isinstance(db_column_type, postgresql.ENUM)
         validate_enum_column_type(metadata, db_column, schema_column)
     else:
         # compare types by using the string version of the types.
         # seems to account for differences btw Integer and INTEGER, String(255) and VARCHAR(255) etc
 
-        assert str(schema_column.type) == str(db_column.type)
+        assert str(schema_column_type) == str(db_column_type)
 
 
 def validate_enum_column_type(metadata, db_column, schema_column):
@@ -1209,14 +1225,49 @@ class TestPostgresRunner(BaseTestRunner):
 
         diff = r2.compute_changes()
 
+        exp_message = 'drop column meaning_of_life from table accounts'
+        assert r2.revision_message() == exp_message
         assert len(diff) == 1
 
-        assert r2.revision_message() == 'drop column meaning_of_life'
+        d = Diff(diff)
+        changes = d.changes()
+        assert len(changes) == 1
+        l = changes.get('accounts')
+        exp_change = {
+            'change': ChangeType.DROP_COLUMN,
+            'desc': exp_message,
+            'col': 'meaning_of_life',
+        }
+        assert len(l) == 1
+        assert l[0] == exp_change
+        d2 = Diff(diff, group_by_table=False)
+        assert d2.list_changes() == [exp_change]
 
         r2.run()
         assert_num_files(r2, 2)
         validate_metadata_after_change(r2, new_metadata)
 
+    @pytest.mark.usefixtures("metadata_with_table")
+    def test_add_multiple_cols(self, new_test_runner, metadata_with_table):
+        r = new_test_runner(metadata_with_table)
+        run_and_validate_with_standard_metadata_tables(r, metadata_with_table)
+
+        new_metadata = conftest.metadata_with_cols_added_to_table(
+            metadata_with_table)
+        new_metadata.bind = r.get_connection()
+        r2 = new_test_runner(new_metadata, r)
+
+        diff = r2.compute_changes()
+
+        d = Diff(diff)
+        changes = d.changes()
+        assert len(changes) == 1
+        l = changes.get('accounts')
+        assert len(l) == 2
+        for change in l:
+            assert change['change'] == ChangeType.ADD_COLUMN
+            assert change['col'] in ['new_column', 'rainbow']
+
     @pytest.mark.usefixtures("metadata_with_table")
     def test_new_enum_column_added_then_removed(self, new_test_runner, metadata_with_table):
         r = new_test_runner(metadata_with_table)
@@ -1231,6 +1282,24 @@ class TestPostgresRunner(BaseTestRunner):
 
         assert len(diff) == 2
 
+        d = Diff(diff)
+        changes = d.changes()
+        assert len(changes) == 2
+        l = changes.get('accounts')
+        assert len(l) == 1
+        assert l[0] == {
+            'change': ChangeType.ADD_COLUMN,
+            'desc': 'add column rainbow to table accounts',
+            'col': 'rainbow',
+        }
+
+        l2 = changes.get('enum_schema')
+        assert len(l2) == 1
+        assert l2[0] == {
+            'change': ChangeType.ADD_ENUM,
+            'desc': 'add enum rainbow_type',
+        }
+
         add_enum = [op for op in diff if isinstance(op, ops.AddEnumOp)]
         modify_table_ops = [op for op in diff if isinstance(
             op, alembicops.ModifyTableOps)]
@@ -1283,6 +1352,12 @@ class TestPostgresRunner(BaseTestRunner):
         assert_num_files(r2, 2)
         validate_metadata_after_change(r2, new_metadata)
 
+    @pytest.mark.usefixtures("metadata_with_arrays")
+    def test_tables_with_arrays(self, new_test_runner, metadata_with_arrays):
+        r = new_test_runner(metadata_with_arrays)
+        run_and_validate_with_standard_metadata_tables(
+            r, metadata_with_arrays, new_table_names=['tbl'])
+
 
 class TestSqliteRunner(BaseTestRunner):
     pass
